rm(list = ls())
#------------ librarys ------------#
library(doParallel)
library(foreach)
library(assist)
library(wavethresh)
library(gss)

#------------ source functions ----------#
source("with_pstd.R")

#------------ parameters set ------------#
# For different function need to change the seed
registerDoParallel(cores = 20)
noise.seed <- 831
n <- 10000
nsim <- 100
nsub <- 1000
s <- 0.1
method <- "m" # change to "v" for gcv & save result in different directory
gss_alpha <- 1 # for gcv
sample.seed <- seq(338,len=nsim)

getDoParWorkers()

#------------ data generation -----------#
x <- seq(0,1,len=n)
f <- doppler(x)
set.seed(noise.seed)
z <- matrix(f+s*rnorm(n*nsim),n,nsim)

#------------- result object -----------#
p1 <- p2 <- p3 <- p4 <- p5o10 <- p5o20 <-p5o30 <- p6 <- p7 <-NULL
se1 <- se2 <- se3 <- se4 <- se5o10 <- se5o20 <-se5o30 <- se6 <- se7 <-NULL
time1 <- time2 <- time3 <- time4 <-time5o10 <- time5o20 <- time5o30 <-time6 <- time7 <-NULL



#------------ 1. ssr all ------------ #
for (i in 1:nsim){
  ptm <- proc.time()
  tmp <- ssr(z[,i]~x,cubic(x),spar=method)
  pred <-predict(tmp)
  p1 <- cbind(p1,pred$fit)
  se1 <- cbind(se1,pred$pstd)
  elapsed <-proc.time()-ptm
  time1 <- c(time1,elapsed[3])
}

#------------ 2.rdr ------------ #
for (i in 1:nsim){
  print(i)
  ptm <- proc.time()
  fit_rdr <-rdr(x,z[,i],nsub,spar=method,seed=sample.seed[i])
  p2 <- cbind(p2,fit_rdr$fit)
  se2 <- cbind(se2,fit_rdr$std)
  elapsed<-proc.time()-ptm
  time2 <-c(time2,elapsed[3])
}


#------------ 3. drdr ------------ #
for (i in 1:nsim){
  print(i)
  ptm <- proc.time()
  fit_drdr <-drdr(x,z[,i],nsub,spar=method,seed=sample.seed[i])
  p3 <- cbind(p3,fit_drdr$fit)
  se3 <- cbind(se3,fit_drdr$std)
  elapsed<-proc.time()-ptm
  time3 <-c(time3,elapsed[3])
}


#------------ 4.sdr ------------ #
for (i in 1:nsim){
  print(i)
  ptm <- proc.time()
  fit_sdr <- sdr(x,z[,i],nsub,spar=method)
  p4 <- cbind(p4,fit_sdr$fit)
  se4 <- cbind(se4,fit_sdr$std)
  elapsed<-proc.time()-ptm
  time4 <-c(time4,elapsed[3])
}

#------------ 5.osdr overlap 0.1------------ #
overlap.rate <- 0.1 # for osdr
for (i in 1:nsim){
  print(i)
  ptm <- proc.time()
  fit_osdr<-osdr(x,z[,i],nsub,overlap=overlap.rate,spar=method)
  p5o10 <- cbind(p5o10,fit_osdr$fit)
  se5o10 <- cbind(se5o10,fit_osdr$std)
  elapsed<-proc.time()-ptm
  time5o10 <-c(time5o10,elapsed[3])
}


#------------ 5.osdr overlap 0.2------------ #
overlap.rate <- 0.2 # for osdr
for (i in 1:nsim){
  print(i)
  ptm <- proc.time()
  fit_osdr<-osdr(x,z[,i],nsub,overlap=overlap.rate,spar=method)
  p5o20 <- cbind(p5o20,fit_osdr$fit)
  se5o20 <- cbind(se5o20,fit_osdr$std)
  elapsed<-proc.time()-ptm
  time5o20 <-c(time5o20,elapsed[3])
}

#------------ 5.osdr overlap 0.3------------ #
overlap.rate <- 0.3 # for osdr
for (i in 1:nsim){
  print(i)
  ptm <- proc.time()
  fit_osdr<-osdr(x,z[,i],nsub,overlap=overlap.rate,spar=method)
  p5o30 <- cbind(p5o30,fit_osdr$fit)
  se5o30 <- cbind(se5o30,fit_osdr$std)
  elapsed<-proc.time()-ptm
  time5o30 <-c(time5o30,elapsed[3])
}

#------------ 6.ubs ------------#
result6 <-foreach(i=1:nsim,.combine=cbind) %dopar%{
  library(gss)
  ptm <- proc.time()
  pred <- ubs(x,z[,i],method=method,alpha=gss_alpha)
  elapsed <-proc.time()-ptm
  c(elapsed[3],pred$fit,pred$se.fit)
}

time6 <-result6[1,]
p6 <-result6[2:(n+1),]
se6 <- result6[(n+2):(2*n+1),]

#------------ 7.abs ------------#
result7 <-foreach(i=1:nsim,.combine=cbind) %dopar%{
  library(gss)
  ptm <- proc.time()
  pred <- abs(x,z[,i],method=method,alpha=gss_alpha)
  elapsed <-proc.time()-ptm
  c(elapsed[3],pred$fit,pred$se.fit)
}

time7 <-result7[1,]
p7 <-result7[2:(n+1),]
se7 <- result7[(n+2):(2*n+1),]


#----------- Save Result ----------#
save(x, f, 
     p1, p2, p3, p4, p5o10, p5o20, p5o30, p6, p7,
     se1, se2, se3, se4, se5o10, se5o20, se5o30, se6, se7,
     time1, time2, time3, time4, time5o10, time5o20, time5o30, time6, time7,
     file="gml_doppler_s1.rda")

#------------------------ tables and plot ----------------------#
nvar <- function(x){
  sum((x-mean(x))^2)/length(x)
}

#-------------- Find typical fit data ----------------#
drop.na_p6 <-as.vector(which(colSums(is.na(p6))>0))
drop.na_p7 <-as.vector(which(colSums(is.na(p7))>0))
exc <- sort(unique(c(drop.na_p6,drop.na_p7)))
set.seed(9074)
if(length(exc)>0){s1 <-sample((1:nsim)[-exc],1)}else{s1 <-sample(1:nsim,1)}
p1.1=p1[,s1];p2.1=p2[,s1];p3.1=p3[,s1]
p4.1=p4[,s1];p5o20.1=p5o20[,s1];p6.1=p6[,s1];p7.1=p7[,s1]

#--------------- plot of typical fit -----------------#
p <- list(p1.1,p2.1,p3.1,p5o20.1,p6.1,p7.1)
title <- c("ALL","RDR","DRDR","OSDR-20%","UBS","ABS")
color <-c(2,3,4,5,6,"gold")
pdf(file="spline-typical-fit-doppler-s1.pdf",height=9,width=8)
par(mfrow=c(3,2),mgp=c(2,1,0), mar=c(3,3,2,1)+.1)
for (i in 1:6){
  plot(x,f,ylab="",ylim = range(c(p,f)),type="l")
  lines(x,p[[i]],col=2)
  mtext(title[i])
}
dev.off()

#--------------- Remove non convergent simulation result if needed -------------------#

drop.nacol <- function(data){
  drop.na <-which(colSums(is.na(data))>0)
  if (length(drop.na)>0){
    result <- data[,-drop.na]
    return(result)} else {return(data)}
}
p6 <-drop.nacol(p6)
se6 <-drop.nacol(se6)
p7 <-drop.nacol(p7)
se7 <-drop.nacol(se7)

#-------------------------- Compute Bayesian CI --------------------------#

ci.bayes1 <- ci.bayes2<-ci.bayes3<-ci.bayes3<-ci.bayes4<-ci.bayes5o10 <-ci.bayes5o20 <-ci.bayes5o30<- array(NA,c(n,nsim,2))
ci.bayes6<- array(NA,c(n,dim(p6)[2],2))
ci.bayes7<-array(NA,c(n,dim(p7)[2],2))

ci.bayes1[,,1]<-p1-1.96*se2
ci.bayes2[,,1]<-p2-1.96*se2
ci.bayes3[,,1]<-p3-1.96*se3
ci.bayes4[,,1]<-p4-1.96*se4
ci.bayes5o10[,,1]<-p5o10-1.96*se5o10
ci.bayes5o20[,,1]<-p5o20-1.96*se5o20
ci.bayes5o30[,,1]<-p5o30-1.96*se5o30
ci.bayes6[,,1]<-p6-1.96*se6
ci.bayes7[,,1]<-p7-1.96*se7

ci.bayes1[,,2]<-p1+1.96*se2
ci.bayes2[,,2]<-p2+1.96*se2
ci.bayes3[,,2]<-p3+1.96*se3
ci.bayes4[,,2]<-p4+1.96*se4
ci.bayes5o10[,,2]<-p5o10+1.96*se5o10
ci.bayes5o20[,,2]<-p5o20+1.96*se5o20
ci.bayes5o30[,,2]<-p5o30+1.96*se5o30
ci.bayes6[,,2]<-p6+1.96*se6
ci.bayes7[,,2]<-p7+1.96*se7

acp.bayes1 <- apply(ci.bayes1[,,1]<=f & ci.bayes1[,,2]>=f,2,mean)
acp.bayes2 <- apply(ci.bayes2[,,1]<=f & ci.bayes2[,,2]>=f,2,mean)
acp.bayes3 <- apply(ci.bayes3[,,1]<=f & ci.bayes3[,,2]>=f,2,mean)
acp.bayes4 <- apply(ci.bayes4[,,1]<=f & ci.bayes4[,,2]>=f,2,mean)
acp.bayes5_o10 <- apply(ci.bayes5o10[,,1]<=f & ci.bayes5o10[,,2]>=f,2,mean)
acp.bayes5_o20 <- apply(ci.bayes5o20[,,1]<=f & ci.bayes5o20[,,2]>=f,2,mean)
acp.bayes5_o30 <- apply(ci.bayes5o30[,,1]<=f & ci.bayes5o30[,,2]>=f,2,mean)
acp.bayes6 <- apply(ci.bayes6[,,1]<=f & ci.bayes6[,,2]>=f,2,mean)
acp.bayes7 <- apply(ci.bayes7[,,1]<=f & ci.bayes7[,,2]>=f,2,mean)

pcov.bayes1 <- apply(ci.bayes1[,,1]<=f & ci.bayes1[,,2]>=f,1,mean)
pcov.bayes2 <- apply(ci.bayes2[,,1]<=f & ci.bayes2[,,2]>=f,1,mean)
pcov.bayes3 <- apply(ci.bayes3[,,1]<=f & ci.bayes3[,,2]>=f,1,mean)
pcov.bayes4 <- apply(ci.bayes4[,,1]<=f & ci.bayes4[,,2]>=f,1,mean)
pcov.bayes5_o10 <- apply(ci.bayes5o10[,,1]<=f & ci.bayes5o10[,,2]>=f,1,mean)
pcov.bayes5_o20 <- apply(ci.bayes5o20[,,1]<=f & ci.bayes5o20[,,2]>=f,1,mean)
pcov.bayes5_o30 <- apply(ci.bayes5o30[,,1]<=f & ci.bayes5o30[,,2]>=f,1,mean)
pcov.bayes6 <- apply(ci.bayes6[,,1]<=f & ci.bayes6[,,2]>=f,1,mean)
pcov.bayes7 <- apply(ci.bayes7[,,1]<=f & ci.bayes7[,,2]>=f,1,mean)

mse1 <- apply((p1-f)**2,2,mean)
mse2 <- apply((p2-f)**2,2,mean)
mse3 <- apply((p3-f)**2,2,mean)
mse4 <- apply((p4-f)**2,2,mean)
mse5_o10 <- apply((p5o10-f)**2,2,mean)
mse5_o20 <- apply((p5o20-f)**2,2,mean)
mse5_o30 <- apply((p5o30-f)**2,2,mean)
mse6 <- apply((p6-f)**2,2,mean)
mse7 <- apply((p7-f)**2,2,mean)


var1 <-apply(p1,1,nvar)
var2 <-apply(p2,1,nvar)
var3 <-apply(p3,1,nvar)
var4 <-apply(p4,1,nvar)
var5_o10 <-apply(p5o10,1,nvar)
var5_o20 <-apply(p5o20,1,nvar)
var5_o30 <-apply(p5o30,1,nvar)
var6 <-apply(p6,1,nvar)
var7 <-apply(p7,1,nvar)

biasq1<-(apply(p1,1,mean)-f)^2
biasq2<-(apply(p2,1,mean)-f)^2
biasq3<-(apply(p3,1,mean)-f)^2
biasq4<-(apply(p4,1,mean)-f)^2
biasq5_o10 <-(apply(p5o10,1,mean)-f)^2
biasq5_o20 <-(apply(p5o20,1,mean)-f)^2
biasq5_o30 <-(apply(p5o30,1,mean)-f)^2
biasq6<-(apply(p6,1,mean)-f)^2
biasq7<-(apply(p7,1,mean)-f)^2


acp.doppler_s1 <- c(ALL=mean(acp.bayes1),RDR=mean(acp.bayes2),DRDR=mean(acp.bayes3),
                    SDR=mean(acp.bayes4),
                    OSDR_o10=mean(acp.bayes5_o10), OSDR_o20=mean(acp.bayes5_o20), OSDR_o30=mean(acp.bayes5_o30),
                    UBS=mean(acp.bayes6),ABS=mean(acp.bayes7))

mse.doppler_s1 <- c(ALL=mean(mse1),
                    RDR=mean(mse2),DRDR=mean(mse3),
                    SDR=mean(mse4),
                    OSDR_o10=mean(mse5_o10), OSDR_o20=mean(mse5_o20), OSDR_o30=mean(mse5_o30),
                    UBS=mean(mse6),ABS=mean(mse7))

var.doppler_s1 <- c(ALL=mean(var1),
                    RDR=mean(var2),DRDR=mean(var3),
                    SDR=mean(var4),
                    OSDR_o10=mean(var5_o10), OSDR_o20=mean(var5_o20), OSDR_o30=mean(var5_o30),
                    UBS=mean(var6),ABS=mean(var7))

biasq.doppler_s1 <- c(ALL=mean(biasq1),
                      RDR=mean(biasq2),DRDR=mean(biasq3),
                      SDR=mean(biasq4),
                      OSDR_o10=mean(biasq5_o10), OSDR_o20=mean(biasq5_o20), OSDR_o30=mean(biasq5_o30),
                      UBS=mean(biasq6),ABS=mean(biasq7))


gml_doppler_case_s1 <- cbind(biasq.doppler_s1,var.doppler_s1,mse.doppler_s1)







